"""
Taktile Auth
"""
import pkg_resources

__version__ = pkg_resources.get_distribution(
    __name__.split(".", maxsplit=1)[0]
).version

from taktile_auth.client import AuthClient  # noqa: 401
from taktile_auth.exceptions import (  # noqa: 401
    InsufficientRightsException,
    InvalidAuthException,
    TaktileAuthException,
)
from taktile_auth.schemas.session import SessionState  # noqa: 401
from taktile_auth.schemas.token import TaktileIdToken  # noqa: 401

__all__ = [
    "AuthClient",
    "InsufficientRightsException",
    "InvalidAuthException",
    "SessionState",
    "TaktileAuthException",
    "TaktileIdToken",
]



import pathlib

import pkg_resources
from pydantic import BaseSettings


class Settings(BaseSettings):
    ENV: str = "local"
    RESOURCE_PATH: pathlib.Path = pathlib.Path(
        pkg_resources.resource_filename(
            "taktile_auth", "assets/resources.yaml"
        )
    )
    ROLE_PATH: pathlib.Path = pathlib.Path(
        pkg_resources.resource_filename("taktile_auth", "assets/roles.yaml")
    )

    CACHE_SPEEDUP_TIME_MINUTES: int = 60
    CACHE_FALLBACK_TIME_MINUTES: int = 72 * 60
    CACHE_PUBLIC_KEY_SPEEDUP_TIME_MINUTES: int = 4 * 60
    CACHE_PUBLIC_KEY_FALLBACK_TIME_MINUTES: int = 10 * 24 * 60
    AUTH_SERVER_TIMEOUT_SECONDS: int = 4


settings = Settings()



from __future__ import annotations

import functools
import re
import typing as t

import pydantic

from taktile_auth.enums import Wildcard

PARTIAL_RE = re.compile(r"(^[^\*]*$)|(^[^\*]*\*$)")


def is_full_wildcard(v: str) -> str:
    assert v == "*" or "*" not in v
    return v


def is_partial_wildcard(v: str) -> str:
    assert re.fullmatch(PARTIAL_RE, v)
    return v


def is_fully_specified(v: str) -> str:
    assert "*" not in v
    return v


WILDCARD_CHECK = {
    Wildcard.allowed: is_full_wildcard,
    Wildcard.partial: is_partial_wildcard,
    Wildcard.not_allowed: is_fully_specified,
}


class Resource(pydantic.BaseModel):
    def __contains__(self, o: Resource) -> bool:
        """Checks if the queried resource 'o' is either equal
        or contained inside of 'self'.
        For each arg of the resource, checks if it is either a
        match with query's corresponding arg (including wildcards).
        """

        if self.schema()["title"] != o.schema()["title"]:
            return False

        for arg in self.dict().keys():
            allowed = getattr(self, arg)
            queried = getattr(o, arg)
            if not re.fullmatch(allowed.replace("*", ".*"), queried):
                return False
        return True

    def __hash__(self) -> int:
        return hash((type(self).__name__,) + tuple(self.__dict__.values()))

    class Config:
        allow_mutation = False


class ResourceDefinition(pydantic.BaseModel):
    resource_name: str
    args: t.Dict[str, Wildcard]

    def get_resource(self) -> t.Type[Resource]:
        return make_resource(
            resource_name=self.resource_name, args=HDict(self.args)
        )


class HDict(dict):  # type: ignore
    def __hash__(self) -> int:  # type: ignore
        return hash(frozenset(self.items()))


# performance counter probe
resources_built = 0


@functools.lru_cache(512)
def make_resource(resource_name: str, args: HDict) -> t.Type[Resource]:
    """
    Builds a resource class with the given name and args.
    Dynamic Pydantic model building is *really* expensive, hence we
    have this rubbish to save on it.
    See AUTH-1669 for the performance outcomes
    """
    global resources_built
    resources_built += 1
    fields: t.Any = {field_name: (str, ...) for field_name in args.keys()}
    validators = {
        f"{field_name}_validator": (
            pydantic.validator(field_name, allow_reuse=True)(
                WILDCARD_CHECK[check]
            )
        )
        for field_name, check in args.items()
    }
    return t.cast(
        t.Type[Resource],
        pydantic.create_model(
            resource_name,
            **fields,
            __validators__=validators,
            __base__=Resource,
        ),
    )



import typing as t


class MemoryCache:
    """Only for testing purposes"""

    def __init__(self) -> None:
        self._cache: t.Dict[str, str] = {}

    def get(self, key: str) -> t.Optional[str]:
        return self._cache.get(key)

    def put(
        self, key: str, value: str, time_to_live: t.Optional[int] = None
    ) -> None:
        del time_to_live
        self._cache[key] = value

    def delete(self, key: str) -> None:
        if key in self._cache:
            del self._cache[key]



import pathlib
import typing as t

import yaml

from taktile_auth.entities import (
    PermissionDefinition,
    ResourceDefinition,
    RoleDefinition,
)
from taktile_auth.parser.utils import parse_body


def parse_role(
    role_name: str,
    role_yaml: t.Dict[str, t.Any],
    resources: t.Dict[str, ResourceDefinition],
) -> RoleDefinition:
    perm_definitions = []
    sub_role_definitions = []
    role_args = role_yaml[role_name]["role_args"]
    role_body = role_yaml[role_name]["role_body"]
    clauses = parse_body(role_body)
    for clause in clauses:
        if clause["type"] == "permission":
            perm_definitions.append(
                (
                    set(clause["actions"]),
                    PermissionDefinition(
                        resource_definition=resources[clause["resource_name"]],
                    ),
                )
            )
        else:
            sub_role_definitions.append(
                parse_role(clause["sub_role_name"], role_yaml, resources)
            )
    return RoleDefinition(
        name=role_name,
        permission_definitions=perm_definitions,
        args=role_args,
        sub_role_definitions=sub_role_definitions,
    )


def parse_resource_yaml(path: pathlib.Path) -> t.Dict[str, ResourceDefinition]:
    with open(path, "r") as f:
        resources_yaml = yaml.safe_load(f)
    resources: t.Dict[str, ResourceDefinition] = {}
    for resource_name, args in resources_yaml.items():
        resources[resource_name] = ResourceDefinition(
            resource_name=resource_name, args=args
        )
    return resources


def parse_role_yaml(
    role_path: pathlib.Path, resource_path: pathlib.Path
) -> t.Dict[str, RoleDefinition]:
    role_yaml = {}
    resources = parse_resource_yaml(resource_path)
    with open(role_path, "r") as f:
        roles: t.Dict[str, RoleDefinition] = yaml.safe_load(f)

    for role_signature, role_body in roles.items():
        role_name, _, role_args_str = role_signature.partition("/")
        role_args = role_args_str.split(",") if role_args_str else []
        role_yaml[role_name] = {"role_args": role_args, "role_body": role_body}

    return {
        role_name: parse_role(role_name, role_yaml, resources)
        for role_name in role_yaml.keys()
    }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['taktile_auth',
 'taktile_auth.entities',
 'taktile_auth.parser',
 'taktile_auth.schemas',
 'taktile_auth.test_utils',
 'taktile_auth.utils']

package_data = \
{'': ['*'], 'taktile_auth': ['assets/*']}

install_requires = \
['PyJWT[crypto]==2.4.0',
 'PyYAML>=6.0,<7.0',
 'cryptography>=38.0.3,<42.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'requests==2.31.0']

setup_kwargs = {
    'name': 'taktile-auth',
    'version': '0.0.35',
    'description': 'Auth Package for Taktile',
    'long_description': '# Taktile Auth\n\n[![pypi status](https://img.shields.io/pypi/v/taktile-auth.svg)](https://pypi.python.org/pypi/taktile-auth)\n[![License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://www.apache.org/licenses/LICENSE-2.0)\n\nThis package is part of the Taktile ecosystem.\n\nTaktile enables data science teams to industrialize, scale, and maintain machine learning models. Our ML development platform makes it easy to create your own end-to-end ML applications:\n\n- Turn models into auto-scaling APIs in a few lines of code\n- Easily add model tests\n- Create and share model explanations through the Taktile UI\n\nFind more information in our [docs](https://docs.taktile.com).\n',
    'author': 'Taktile GmbH',
    'author_email': 'devops@taktile.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
